@@ -2,12 +2,14 @@ from django.core.management.base import AppCommand, CommandError
 from django.db.models import get_models
 from django.template import loader as TemplateLoader, Context
 from optparse import make_option
+from bricklayer.utils import ColoredOutputMixin
 import os
 import re
+import imp
 
-
-class Command(AppCommand):
-    help = "Creates a basic admin.py file if it doesn't exists. For each model, create an AdminModel class (if it doesn't exists) and then register the model on the admin site."
+class Command(AppCommand,ColoredOutputMixin):
+    help = """Creates and populates a basic admin.py file if it doesn't exists. For each model in the app, 
+            create an AdminModel class (if it doesn't exists) and then register the model on the admin site."""
 
     args = '[appname ...]'
 
@@ -20,9 +22,11 @@ class Command(AppCommand):
     )
 
     def handle_app(self, app, **options):
-        app_name = app.__name__.split('.')[-2]
-
+        app_label = app.__name__.split('.')[-2]
+        app_path = os.path.dirname(app.__file__)
         admin_filename = os.path.join(os.path.dirname(app.__file__), 'admin.py')
+        is_verbose = options['verbosity'] >= 1
+        
         # Get the name of each model in the app
         app_models = [model._meta.object_name for model in get_models(app)]
         # if models    
@@ -30,17 +34,17 @@ class Command(AppCommand):
         if options['models']:
             # Accept list of models as CSV
             requested_models = options["models"].split(",")
-            for model in requested_models:
-                if model not in app_models:
-                    raise CommandError("No model %s in %s"%(model,app_name))
             app_models = filter(lambda model: model in requested_models, app_models)
+        
         if not os.path.exists(admin_filename):
-            self.stdout.write(app_name+".admin.py created")
+            self.stdout.write(app_label+".admin.py created")
             mode = "w"
-            context = Context({"app": app_name, 
+            context = Context({"app": app_label, 
+                               "import_admin": True,
                                 "models": app_models,
                                 "models_to_define": app_models,
-                                "models_to_register": app_models})
+                                "models_to_register": app_models,
+                                "models_to_import":app_models})
             
         else:
             self.stdout.write("admin.py already exists")
@@ -49,7 +53,7 @@ class Command(AppCommand):
             For each model which hasn't been registered yet,
             create an AdminClass and register the model.
             """
-            
+            admin_module = imp.load_source("admins",os.path.join(app_path, 'admins.py'))
             with open(admin_filename, 'r') as admin_file:
                 admin_file_lines = admin_file.readlines()
                 admin_file_text = "".join( admin_file_lines  )
@@ -57,9 +61,14 @@ class Command(AppCommand):
             mode = 'a'
             models_to_define = []
             models_to_register = []
+            models_to_import = []
             for model in app_models:
                 regex = re.compile("(admin.site.register\(\s*"+model+"\s*,)",re.MULTILINE)
                 
+                #if them model wasn't loaded yet...
+                if model not in dir(admin_module):
+                    models_to_import.append(model)
+                    
                 #if model has not been registered yet..
                 if not regex.search(admin_file_text):
                     models_to_register.append(model)
@@ -68,12 +77,21 @@ class Command(AppCommand):
                 if not "%sAdmin"%model in admin_file_text:
                     models_to_define.append(model)
             
-            context = Context({"app": app_name, 
-                              "models": app_models,
-                              "models_to_define": models_to_define,
-                              "models_to_register": models_to_register})
+            context = Context({"app": app_label,
+                               "import_admin": "admin" in dir(admin_module),
+                               "models": app_models,
+                               "models_to_define": models_to_define,
+                               "models_to_import":models_to_import,
+                               "models_to_register": models_to_register})
 
         # Write admin.py   
         with open(admin_filename, mode) as admin_file:
-                admin_file.write(template.render(context))    
+                admin_file.write(template.render(context))
+        
+        if is_verbose:
+            if mode == "w":
+                msg = "%s.admins created"
+            else:
+                msg = "%s.admins updated"
+            self.print_success(msg%app_label)
             
\ No newline at end of file
