@@ -1,9 +1,9 @@
 from django.conf import settings
-from django.core.management.base import AppCommand, CommandError
+from django.core.management.base import AppCommand
 from django.db.models import get_models, get_model
 from django.template import loader as TemplateLoader, Context
 from django.utils.module_loading import import_module
-from bricklayer.utils import exists_dir, exists_path
+from bricklayer.utils import exists_dir, exists_path, is_blank
 from optparse import make_option
 import os, imp
 import inspect
@@ -18,45 +18,72 @@ class Command(AppCommand):
                     type="string",
                     default=None,
                     help='Model/s name. Multiple models can be passed separated by a comma. E.g. -m User,Post,Comment'),
-        make_option('--all', '-a', 
-                    dest='all',
-                    action="store_true",
-                    default=False,
-                    help='Process all models'),
         make_option('--form', '-f', 
                     dest='forms',
                     action="store_true",
                     default=False,
                     help='Generate a ModelForm for each model'),
+        make_option('-c', 
+                    dest='create',
+                    action="store_true",
+                    default=False,
+                    help='Generate only the create view'),
+        make_option('-r', 
+                    dest='read',
+                    action="store_false",
+                    default=True,
+                    help='Generate only the detail and list views'),
+        make_option('-u', 
+                    dest='update',
+                    action="store_false",
+                    default=True,
+                    help='Generate only the update view'),
+        make_option('-d', 
+                    dest='delete',
+                    action="store_false",
+                    default=True,
+                    help='Generate only the delete'),
+                                            
+                                        
     )
     
 
     def handle_app(self, app, **options):
                 
-        
-        if options['all'] and options['models']:
-            raise CommandError("Do you want process all models or just %s?"%options['models'])
-        
         # Check if multiple model are passed as arguments
         if options['models'] and ',' in options['models']:
             options['models'] = options['models'].split(',')
         else:
             options['models'] = [ options['models'] ]
         
+        is_verbose = options['verbosity'] >= 1
         # some useful shortcuts
         app_models = [model._meta.object_name for model in get_models(app)]
-        models = app_models if options['all'] else options['models']
+        models = options['models'] or app_models
         app_name = app.__name__.split('.')[-2]
         app_path = os.path.dirname(app.__file__)
         template_path = os.path.join(app_path, 'templates')
         app_template_path = os.path.join(app_path, 'templates', app_name)
        
         # Test if models exists in the app
-        if not options['all']:
-            for model in models:
-                if model not in app_models:
-                    raise CommandError("No model %s in %s"%(model,app_name))
+        if options['models']:
+            models = [ model for model in options['models'] if model in app_models ]
+        
+        # Set which views are going to be generated
+        generic_views = ["CreateView", "DetailView", "ListView", "UpdateView", "DeleteView"]
+        if options['create']:
+            generic_views.remove("CreateView")
+        
+        if options['read']:
+            generic_views.remove("DetailView")
+            generic_views.remove("ListView")
+        
+        if options['update']:
+            generic_views.remove("UpdateView")
             
+        if options['delete']:
+            generic_views.remove("DeleteView")
+                
         # Create templates folder if not exists
         if not exists_dir(template_path):
             os.mkdir(template_path)
@@ -110,30 +137,19 @@ class Command(AppCommand):
             
             # Create forms.py
             if options["forms"]:
-                template = TemplateLoader.get_template("forms.py.template")
-                if not exists_path(app_path, 'forms.py'):
-                    mode = "w"
-                    context = Context({'model': model,
-                                        'app': app_name,
-                                        'import_forms': True,
-                                        'already_defined': False})
-                else:
-                    forms = imp.load_source("views",os.path.join(app_path, 'forms.py'))
-                    mode = "a"
-                    context = Context({'app': app_name,
-                                       'model': model,
-                                       'import_forms': not hasattr(forms, "forms"),
-                                       'already_defined':  hasattr(forms, model+"Form")})
-                with open(os.path.join(app_path, "forms.py"), mode) as f:
-                    f.write(template.render(context))
-                
+                from bricklayer.management.commands import GenerateFormCommand
+                import copy
+                command = GenerateFormCommand()
+                form_options = copy.copy(options)
+                form_options["models"] = model
+                command.handle_app(app, **form_options)
                 
             # Create views.py
             # If views.py exists (it should exists), it generates a view
             # only if one with that name doesn't already exist.
             template = TemplateLoader.get_template("views.py.template")
-            generic_views = ["CreateView", "DetailView", "ListView", "UpdateView", "DeleteView"]
-            if not exists_path(app_path, 'views.py'):
+            
+            if not exists_path(app_path, 'views.py') or is_blank(app_path, "views.py"):
                 mode = "w"
                 context  = Context({'app': app_name,
                                     'model': model,
@@ -148,21 +164,21 @@ class Command(AppCommand):
                                     'update': True,
                                     'delete': True})
             else:
-                views = imp.load_source("views",os.path.join(app_path, 'views.py'))
+                views_module = dir( imp.load_source("views",os.path.join(app_path, 'views.py')) )
                 mode ="a"
-                generic_views = filter(lambda view: view not in dir(views), generic_views)
+                generic_views = filter(lambda view: view not in views_module, generic_views)
                 context  = Context({'app': app_name,
                                     'model': model,
-                                    'import_model': not hasattr(views, model),
-                                    'import_reverse_lazy': not hasattr(views, "reverse_lazy" ),
-                                    'import_form': options['forms'] and not hasattr(views, model+"Form" ),
+                                    'import_model': model not in views_module,
+                                    'import_reverse_lazy': "reverse_lazy" not in views_module ,
+                                    'import_form': options['forms'] and model+"Form" not in views_module ,
                                     'has_form': options['forms'],
                                     'generic_views': generic_views,
-                                    'create': not hasattr(views, model+"Create" ),
-                                    'detail': not hasattr(views, model+"Detail" ),
-                                    'list':   not hasattr(views, model+"List" ),
-                                    'update': not hasattr(views, model+"Update" ),
-                                    'delete': not hasattr(views, model+"Delete" )})
+                                    'create': model+"Create" not in views_module,
+                                    'detail': model+"Detail" not in views_module,
+                                    'list':   model+"List" not in views_module,
+                                    'update': model+"Update" not in views_module,
+                                    'delete': model+"Delete" not in views_module})
                 
             with open(os.path.join(app_path, 'views.py'),mode) as f:
                     f.write(template.render(context))
@@ -171,23 +187,26 @@ class Command(AppCommand):
             # Create urls.py
             template = TemplateLoader.get_template("urls.py.template")
             imports = ['url', 'patterns']
+            generic_view_types = map(lambda v: v.replace("View",""), generic_views) 
             if not exists_path(app_path, 'urls.py'):
                 context  = Context({'app': app_name,
                                     'model': model,
                                     'imports':imports,
-                                    'extend_urlpatterns': False })
+                                    'extend_urlpatterns': False,
+                                    'generic_view_types':generic_view_types })
                 
                 mode = "w"
             else:
-                urls = imp.load_source("urls",os.path.join(app_path, 'urls.py'))
-                imports = filter( lambda func: func not in dir(urls), imports)
-                extend_urlpatterns = hasattr(urls, 'urlpatterns')
+                urls_module = dir( imp.load_source("urls",os.path.join(app_path, 'urls.py')) )
+                imports = filter( lambda func: func not in urls_module, imports)
+                extend_urlpatterns = 'urlpatterns' in urls_module
                     
                 context  = Context({'app': app_name,
                                     'model_name':model_name, 
                                     'model': model,
                                     'imports':imports,
-                                    'extend_urlpatterns': extend_urlpatterns })
+                                    'extend_urlpatterns': extend_urlpatterns,
+                                    'generic_view_types': generic_view_types })
                 mode = "a"
             
             with open(os.path.join(app_path, 'urls.py'),mode) as f:
@@ -197,11 +216,10 @@ class Command(AppCommand):
             root_urlconf_path = inspect.getsourcefile(import_module(settings.ROOT_URLCONF)) 
             with open(root_urlconf_path, "r") as f:
                 urls_text = "".join(f.readlines())
-            with open(root_urlconf_path, "w") as f:
+            with open(root_urlconf_path, "a") as f:
                 extend_patterns = "\nurlpatterns+=patterns(\"\",url(r\"%s/\",include(\"%s.urls\")),)"%(model_name, app_name)
                 if extend_patterns not in urls_text:
-                    urls_text += extend_patterns
-                f.write(urls_text)
+                    f.write(extend_patterns)
         
             
             
\ No newline at end of file
